-- awesome 3 configuration file

-- Include awesome library, with lots of useful function!
require("awful")
require("awful.autofocus")
require("awful.rules")
require("beautiful")
require("vicious")

terminal = "urxvtcd"
lock     = 'xscreensaver-command -lock'
lock     = 'gnome-screensaver-command --lock'
beautiful.init(awful.util.getdir("config").."/theme")

-- {{{ Modkeys
modkey  = "Mod4"
shift   = "Shift"
alt     = "Mod1"
control = "Control"

k_n   = {}
k_m   = {modkey}
k_ms  = {modkey, shift}
k_ma  = {modkey, alt}
k_mc  = {modkey, control}
k_mcs = {modkey, control, shift}
k_a   = {alt}
k_ac  = {alt, control}
k_c   = {control}
k_cs  = {control, shift}
k_s   = {shift}

-- }}}
-- {{{ Markup helper functions
-- Inline markup is a tad ugly, so use these functions
-- to dynamically create markup.
function fg(color, text)
    return '<span color="'..color..'">'..text..'</span>'
end

-- }}}

-- {{{ Tags

tags = {}
for s = 1, screen.count() do
    -- Each screen has its own tag table.
    local p = {}
    for i = 1, 10 do
        table.insert(p, "work-"..i)
    end
    tags[s] = awful.tag(p, s, awful.layout.suit.tile)
end

-- Get the screen number we're on
local mtag = { }
function mtag.getn(idx, s)
    return tags[s or mouse.screen][idx]
end
function mtag.viewonly (idx, s)
    local t = mtag.getn(idx, s)
    if t then awful.tag.viewonly(t) end
end
function mtag.viewtoggle (idx, s)
    local t = mtag.getn(idx, s)
    if t then awful.tag.viewtoggle(t) end
end
function mtag.isoccupied(s, t)
    local clients = (t and t:clients()) or {}
    return #clients > 0
end
function mtag.occupied(s)
    local p = {}

    if not s then s = mouse.screen end
    for t in pairs(tags[s]) do
        t = tags[s][t]
        if mtag.isoccupied(s, t) then table.insert(p, t) end
    end
    return p
end
function mtag.getnext(s)
    if s == nil then s = mouse.screen end

    local p = mtag.occupied(s)
    local curtag = awful.tag.selected()

    local t = 0

    -- Get tag #
    if mtag.isoccupied(s, curtag) then
        for x in pairs(p) do
            if curtag == p[x] then t = x end
        end
    end

    local lasto = 1
    local o = 0

    for x in pairs(tags[s]) do
        if curtag == tags[s][x] then o = x end
        if mtag.isoccupied(s, tags[s][x]) then lasto = x end
    end

    -- Now: t is # in non-empty, o is # in all
    if o == table.maxn(tags[s]) then
        -- We're the last tag, create a new one
        if t == 0 then
            -- We're empty, go to first
            return tags[s][1]
        else
            -- We're occupied, create new
            return newtag()
        end
    else
        if t == 0 then
            -- We're empty, check if we're last
            if o > lasto then
                -- We're also later than the last non-empty
                -- Wrap to first
                return tags[s][1]
            else
                -- Nevermind, get the next
                return tags[s][o+1]
            end
        else
            -- Return next tag
            return tags[s][o+1]
        end
    end
end
function mtag.next(s)
    awful.tag.viewonly(mtag.getnext(s))
end
function mtag.movetonext(s)
    local t = mtag.getnext(s)
    awful.client.movetotag(t)
    awful.tag.viewonly(t)
end
function mtag.getprev(s)
    if s == nil then s = mouse.screen end

    local p = mtag.occupied(s)
    local curtag = awful.tag.selected()

    local t = 0

    -- Get tag #
    if mtag.isoccupied(s, curtag) then
        for x in pairs(p) do
            if curtag == p[x] then t = x end
        end
    end

    local lasto = 1
    local o = 0

    for x in pairs(tags[s]) do
        if curtag == tags[s][x] then o = x end
        if mtag.isoccupied(s, tags[s][x]) then lasto = x end
    end

    -- Now: t is # in non-empty, o is # in all
    if o == 1 then
        -- We're the very first tag, wrap around
        return p[ table.maxn(p) ]
    else
        -- We're not first, just go prev
        return tags[s][o-1]
    end
end
function mtag.prev(s)
    awful.tag.viewonly(mtag.getprev(s))
end
function mtag.movetoprev(s)
    local t = mtag.getprev(s)
    awful.client.movetotag(t)
    awful.tag.viewonly(t)
end


-- }}}
-- {{{ Widgets

local icondir = awful.util.getdir("config").."/icons/"

function make_icon(fname, left, right, h, bg)
    local icon = image(icondir..fname)
    local ib   = widget { type = 'imagebox' }
    local w    = left + icon.width + right
    local i    = image.argb32(w, h, nil)

    i:draw_rectangle(0, 0, w, h, true, beautiful.bg_normal)
    i:insert(icon, left, math.floor((h - icon.height) / 2))

    ib.image = i
    return ib
end

function draw_dashes_h(img, x, y, on, off, len, color)
    for i = 0, len, on + off do
        img:draw_line(x + i, y, x + i + on - 1, y, color)
    end
end

function draw_dashes_v(img, x, y, on, off, len, color)
    for i = 0, len, on + off do
        img:draw_line(x, y + i, x, y + i + on - 1, color)
    end
end


local mywidgets = {}

-- {{{ Taglist

maintaglist = {}
maintaglist.buttons = awful.util.table.join(
    awful.button(k_n, 1, awful.tag.viewonly),
    awful.button(k_s, 1, awful.client.toggletag)
)
maintaglist.label = awful.widget.taglist.label.noempty

-- }}}
-- {{{ Prompt box

mypromptbox = awful.widget.prompt{
    layout = awful.widget.layout.horizontal.leftright
}

-- }}}
-- {{{ CPU Widgets

local cpuwidgets = {}
for i = 1, io.popen("grep -c ^cpu /proc/stat"):read("*n") do
    local cpu = {}

    cpu.widget = widget{ type = 'imagebox' }
    cpu.update = function(cpu, value)
        local color

        if value < 25 then
            color = beautiful.fg_focus
        elseif value < 50 then
            color = 'yellow'
        elseif value < 75 then
            color = 'orange'
        else
            color = 'red'
        end

        if cpu.color == color then return end
        local img = image.argb32(11, 16, nil)
        draw_dashes_h(img, 0,  4, 2, 1, 7, color)
        draw_dashes_h(img, 0, 11, 2, 1, 7, color)
        draw_dashes_v(img, 0,  4, 2, 1, 7, color)
        draw_dashes_v(img, 7,  4, 2, 1, 7, color)
        img:draw_rectangle(2, 6, 4, 4, true, color)
        cpu.widget.image = img
        cpu.percent = value
    end
    table.insert(mywidgets, cpu.widget)
    cpuwidgets[i] = cpu
end

cputextwidget = widget{ type = 'textbox' }

vicious.register(cputextwidget, vicious.widgets.cpu,
function (widget, args)
    local r = tonumber(args[1])
    local percent = args[1]..'%'

    for i = 1, #args do
        cpuwidgets[i]:update(args[i])
    end

    if r < 10 then
        percent = '0'..percent
    end
    if r < 25 then
        percent = fg(beautiful.fg_focus, percent)
    elseif r < 50 then
        percent = fg('yellow', percent)
    elseif r < 75 then
        percent = fg('orange', percent)
    else
        percent = fg('red', percent)
    end
    return percent..' '
end, 2)

table.insert(mywidgets, cputextwidget)

-- }}}
-- {{{ Memory Usage Widget

table.insert(mywidgets, make_icon('mem.png', 16, 4, 16))

memtextwidget = widget{ type = 'textbox' }

vicious.register(memtextwidget, vicious.widgets.mem, function (widget, args)
    -- Add extra preceding zeroes when needed
    local r = tonumber(args[1])
    local percent = args[1]..'%'
    if r < 10 then
        percent = '0'..percent
    end
    if r < 50 then
        percent = fg(beautiful.fg_focus, percent)
    elseif r < 80 then
        percent = fg('orange', percent)
    else
        percent = fg('red', percent)
    end
    return percent..' '..args[2]..'M'
end, 2)

table.insert(mywidgets, memtextwidget)

-- }}}
-- {{{ Clock

table.insert(mywidgets, make_icon('clock.png', 16, 4, 16))

clockwidget = widget{ type = "textbox" }
vicious.register(clockwidget, vicious.widgets.date,
                 fg(theme.fg_focus, "%H:%M") .. " %a %d %b  ", 10)

table.insert(mywidgets, clockwidget)

-- }}}
-- {{{ Statusbar

for s = 1, screen.count() do
    local tab = {
        {
            awful.widget.taglist(s, maintaglist.label, maintaglist.buttons),
            mypromptbox.widget,
            layout = awful.widget.layout.horizontal.leftright
        },
    }

    if (s == 1) then
        table.insert(tab, widget{ type = 'systray' })
    end
    for i = #mywidgets, 1, -1 do
        table.insert(tab, mywidgets[i])
    end
    tab["layout"] = awful.widget.layout.horizontal.rightleft
    awful.wibox{ position = "top", height = 16, screen = s }.widgets = tab
end

-- }}}
-- }}}
-- {{{ Keys
---- {{{ Global keys

globalkeys = awful.util.table.join(
    -- Mod+{A/S}: Switch to prev/next tag
    awful.key(k_m, "Left",  mtag.prev),
    awful.key(k_m, "Right", mtag.next),

    -- Mod+Shift+{A/S}: Move window to Prev/Next tag
    awful.key(k_ms, "Left",  mtag.movetoprev),
    awful.key(k_ms, "Right", mtag.movetonext),

    -- Mod+Shift_{E/D}: move window to next/prev screen
    awful.key(k_mc, "Right", function()
       client.focus.screen = awful.util.cycle(screen.count(), mouse.screen + 1)
    end),
    awful.key(k_mc, "Left", function()
       client.focus.screen = awful.util.cycle(screen.count(), mouse.screen - 1)
    end),


    -- Focus Prev/Next window
    awful.key(k_m, "j",
        function ()
            awful.client.focus.byidx(1)
            if client.focus then client.focus:raise() end
        end),
    awful.key(k_m, "k",
        function ()
            awful.client.focus.byidx(-1)
            if client.focus then client.focus:raise() end
        end),

    -- Swap window with the Prev/Next one
    awful.key(k_ms, "j", function () awful.client.swap.byidx(1) end),
    awful.key(k_ms, "k", function () awful.client.swap.byidx(-1) end),

    -- Mod+{E/D}: Switch to next/previous screen
    awful.key(k_m, "Tab",  function ()
        awful.screen.focus_relative(1)
        coords = mouse.coords();
        mouse.coords({ x = coords.x + 5, y = coords.y + 5}, true)
    end),
    awful.key(k_ms, "Tab", function ()
        awful.screen.focus_relative(-1)
        coords = mouse.coords()
        mouse.coords({ x = coords.x + 5, y = coords.y + 5}, true)
    end),

    -- Mod+Enter: Launch a new terminal
    awful.key(k_m,  "e",      function() awful.util.spawn("chromium-browser") end),
    awful.key(k_m,  "Return", function() awful.util.spawn(terminal) end),
    awful.key(k_ac, "r", awesome.restart),
    awful.key(k_m, "F12", function() awful.util.spawn(lock) end),
    awful.key({}, "#148", function() awful.util.spawn("kcalc") end),

    -- Layout manipulation
    awful.key(k_m,  "l", function () awful.tag.incmwfact(0.05) end),
    awful.key(k_m,  "h", function () awful.tag.incmwfact(-0.05) end),
    awful.key(k_ms, "h", function () awful.tag.incnmaster(1) end),
    awful.key(k_ms, "l", function () awful.tag.incnmaster(-1) end),
    awful.key(k_mc, "h", function () awful.tag.incncol(1) end),
    awful.key(k_mc, "l", function () awful.tag.incncol(-1) end),

    -- Menu
    awful.key(k_m, "r", function () mypromptbox:run() end),
    awful.key(k_m, "F4",
        function ()
            awful.prompt.run({ prompt = "Run Lua code: " },
                             mypromptbox.widget,
                             awful.util.eval, nil,
                             awful.util.getdir("cache").."/lua_commands")
    end),

    awful.key({}, "#192", function() mtag.viewonly(1) end),
    awful.key({}, "#193", function() mtag.viewonly(2) end),
    awful.key({}, "#194", function() mtag.viewonly(3) end),
    awful.key({}, "#195", function() mtag.viewonly(4) end),
    awful.key({}, "#196", function() mtag.viewonly(5) end)
)

-- Mod+#: Switch to tag
-- Mod+Shift+#: Toggle tag display
-- Mod+Control+#: Move client to tag
-- Mod+Alt+#: Toggle client on tag

for i = 1, 10 do
    globalkeys = awful.util.table.join(
        globalkeys,
        awful.key(k_m,  i % 10, function() mtag.viewonly(i)   end),
        awful.key(k_ms, i % 10, function() mtag.viewtoggle(i) end),
        awful.key(k_mc, i % 10,
                  function ()
                      if client.focus and tags[client.focus.screen][i] then
                          awful.client.movetotag(tags[client.focus.screen][i])
                      end
                  end),
        awful.key(k_mcs, i % 10,
                  function ()
                      if client.focus and tags[client.focus.screen][i] then
                          awful.client.toggletag(tags[client.focus.screen][i])
                      end
                  end)
    )
end

---- }}}
---- {{{ Client hotkeys / buttons

local clientkeys = awful.util.table.join(
    awful.key(k_m, "i", function (c)
        if mypromptbox.widget.text then
            mypromptbox.widget.text = ""
        else
            mypromptbox.widget.text = "Class: " .. c.class .. " Instance: ".. c.instance .. " Name: " .. c.name
        end
    end),

    -- Client manipulation
    awful.key(k_m, "c", function (c) c:kill() end),
    awful.key(k_m, "o", awful.client.floating.toggle),
    awful.key(k_m, "F11", function (c) c.fullscreen = not c.fullscreen end)
)

local clientbuttons = awful.util.table.join(
    awful.button({ }, 1, function (c) client.focus = c; c:raise() end),
    awful.button(k_a, 1, awful.mouse.client.move),
    awful.button(k_a, 3, awful.mouse.client.resize)
)

---- }}}

root.keys(globalkeys)
-- }}}
-- {{{ Rules

awful.rules.rules = {
    -- All clients will match this rule.
    {
        rule = { },
        properties = {
            border_width = beautiful.border_width,
            border_color = beautiful.border_normal,
            focus   = true,
            keys    = clientkeys,
            buttons = clientbuttons,
            size_hints_honor = false,
        }
    },
    { rule = { class = "MPlayer" },  properties = { floating = true } },
    { rule = { class = "pinentry" }, properties = { floating = true } },
    { rule = { class = "Claws-mail", name = "[no subject] - Compose message" },
      properties = { maximized_vertical = true, floating = true, x = 0 } },
}

-- }}}
-- {{{ Signals

client.add_signal("focus", function (c)
    if not awful.client.ismarked(c) then
        c.border_color = beautiful.border_focus
    end
end)

client.add_signal("unfocus", function (c)
    if not awful.client.ismarked(c) then
        c.border_color = beautiful.border_normal
    end
end)

client.add_signal("manage", function (c, startup)
    c:add_signal("mouse::enter", function(c) client.focus = c end)
end)

-- }}}

awful.util.spawn("xkbcomp -w 0 -R/usr/share/X11/xkb /home/madcoder/.Xkeyboard :0")
awful.util.spawn("xsetroot -cursor_name left_ptr")
